@@ -6,9 +6,10 @@ handle lower level of the config file
 """
 
 import os
-from ruamel.yaml import YAML as yaml
 import glob
 from copy import deepcopy
+from itertools import chain
+from ruamel.yaml import YAML as yaml
 
 # local imports
 from dotdrop.version import __version__ as VERSION
@@ -59,6 +60,7 @@ class CfgYaml:
     key_import_profile_dfs = 'import'
     key_import_sep = ':'
     key_import_ignore_key = 'optional'
+    key_import_fatal_not_found = True
 
     # settings
     key_settings_dotpath = Settings.key_dotpath
@@ -225,7 +227,7 @@ class CfgYaml:
             self.log.dbg('dynvariables: {}'.format(self.ori_dvariables))
 
     def _resolve_dotfile_paths(self):
-        """resolve dotfile paths"""
+        """resolve dotfiles paths"""
         t = Templategen(variables=self.variables,
                         func_file=self.settings[Settings.key_func_file],
                         filter_file=self.settings[Settings.key_filter_file])
@@ -233,29 +235,42 @@ class CfgYaml:
         for dotfile in self.dotfiles.values():
             # src
             src = dotfile[self.key_dotfile_src]
-            if not src:
-                dotfile[self.key_dotfile_src] = ''
-            else:
-                new = t.generate_string(src)
-                if new != src and self.debug:
-                    msg = 'dotfile src: \"{}\" -> \"{}\"'.format(src, new)
-                    self.log.dbg(msg)
-                src = new
-                src = os.path.join(self.settings[self.key_settings_dotpath],
-                                   src)
-                dotfile[self.key_dotfile_src] = self._norm_path(src)
-
+            newsrc = self.resolve_dotfile_src(src, templater=t)
+            dotfile[self.key_dotfile_src] = newsrc
             # dst
             dst = dotfile[self.key_dotfile_dst]
-            if not dst:
-                dotfile[self.key_dotfile_dst] = ''
-            else:
-                new = t.generate_string(dst)
-                if new != dst and self.debug:
-                    msg = 'dotfile dst: \"{}\" -> \"{}\"'.format(dst, new)
-                    self.log.dbg(msg)
-                dst = new
-                dotfile[self.key_dotfile_dst] = self._norm_path(dst)
+            newdst = self.resolve_dotfile_dst(dst, templater=t)
+            dotfile[self.key_dotfile_dst] = newdst
+
+    def resolve_dotfile_src(self, src, templater=None):
+        """resolve dotfile src path"""
+        newsrc = ''
+        if src:
+            new = src
+            if templater:
+                new = templater.generate_string(src)
+            if new != src and self.debug:
+                msg = 'dotfile src: \"{}\" -> \"{}\"'.format(src, new)
+                self.log.dbg(msg)
+            src = new
+            src = os.path.join(self.settings[self.key_settings_dotpath],
+                               src)
+            newsrc = self._norm_path(src)
+        return newsrc
+
+    def resolve_dotfile_dst(self, dst, templater=None):
+        """resolve dotfile dst path"""
+        newdst = ''
+        if dst:
+            new = dst
+            if templater:
+                new = templater.generate_string(dst)
+            if new != dst and self.debug:
+                msg = 'dotfile dst: \"{}\" -> \"{}\"'.format(dst, new)
+                self.log.dbg(msg)
+            dst = new
+            newdst = self._norm_path(dst)
+        return newdst
 
     def _rec_resolve_vars(self, variables):
         """recursive resolve variables"""
@@ -383,6 +398,9 @@ class CfgYaml:
             return profiles
         new = {}
         for k, v in profiles.items():
+            if not v:
+                # no dotfiles
+                continue
             # add dotfiles entry if not present
             if self.key_profile_dotfiles not in v:
                 v[self.key_profile_dotfiles] = []
@@ -568,19 +586,16 @@ class CfgYaml:
         paths = self.settings.get(self.key_import_variables, None)
         if not paths:
             return
-        paths = self._glob_paths(paths)
-        for p in paths:
-            path, fatal_not_found = self._norm_extended_import_path(p)
+        paths = self._resolve_paths(paths)
+        for path in paths:
             if self.debug:
                 self.log.dbg('import variables from {}'.format(path))
             var = self._import_sub(path, self.key_variables,
-                                   mandatory=False,
-                                   fatal_not_found=fatal_not_found)
+                                   mandatory=False)
             if self.debug:
                 self.log.dbg('import dynvariables from {}'.format(path))
             dvar = self._import_sub(path, self.key_dvariables,
-                                    mandatory=False,
-                                    fatal_not_found=fatal_not_found)
+                                    mandatory=False)
             merged = self._merge_dict(dvar, var)
             merged = self._rec_resolve_vars(merged)
             # execute dvar
@@ -592,30 +607,106 @@ class CfgYaml:
         """remove profile variables from dic if found"""
         [dic.pop(k, None) for k in self.prokeys]
 
-    def _norm_extended_import_path(self, path):
-        """normalize imported path and its attribute if any"""
-        fields = path.split(self.key_import_sep)
-        fatal_not_found = True
-        filepath = path
-        if len(fields) > 1 and fields[-1] == self.key_import_ignore_key:
-            fatal_not_found = False
-            filepath = ''.join(fields[:-1])
-        return self._norm_path(filepath), fatal_not_found
+    def _parse_extended_import_path(self, path_entry):
+        """Parse an import path in a tuple (path, fatal_not_found)."""
+        if self.debug:
+            self.log.dbg('parsing path entry {}'.format(path_entry))
+
+        path, _, attribute = path_entry.rpartition(self.key_import_sep)
+        fatal_not_found = attribute != self.key_import_ignore_key
+        is_valid_attribute = attribute in ('', self.key_import_ignore_key)
+        if not is_valid_attribute:
+            # If attribute is not valid it can mean that:
+            # - path_entry doesn't contain the separator, and attribute is set
+            #   to the whole path by str.rpartition
+            # - path_entry contains a separator, but it's in the file path, so
+            #   attribute is set to whatever comes after the separator by
+            #   str.rpartition
+            # In both cases, path_entry is the path we're looking for.
+            if self.debug:
+                self.log.dbg('using attribute default values for path {}'
+                             .format(path_entry))
+            path = path_entry
+            fatal_not_found = self.key_import_fatal_not_found
+        elif self.debug:
+            self.log.dbg('path entry {} has fatal_not_found flag set to {}'
+                         .format(path_entry, fatal_not_found))
+        return path, fatal_not_found
+
+    def _handle_non_existing_path(self, path, fatal_not_found=True):
+        """Raise an exception or log a warning to handle non-existing paths."""
+        error = 'bad path {}'.format(path)
+        if fatal_not_found:
+            raise YamlException(error)
+        self.log.warn(error)
+
+    def _check_path_existence(self, path, fatal_not_found=True):
+        """Check if a path exists, raising if necessary."""
+        if os.path.exists(path):
+            if self.debug:
+                self.log.dbg('path {} exists'.format(path))
+            return path
+
+        self._handle_non_existing_path(path, fatal_not_found)
+        # Explicit return for readability. Anything evaluating to false is ok.
+        return None
+
+    def _process_path(self, path_entry):
+        """Process a path entry to a normalized form.
+
+        This method processed a path entry. Namely it:
+        - Normalizes the path.
+        - Expands globs.
+        - Checks for path existence, taking in account fatal_not_found.
+        This method always returns a list containing only absolute paths
+        existing on the filesystem. If the input is not a glob, the list
+        contains at most one element, otheriwse it could hold more.
+
+        :param path_entry: A path with an optional attribute.
+        :type path_entry: str
+        :return: A list of normalized existing paths, obtained from the input.
+        :rtype: List of str
+        """
+        path, fatal_not_found = self._parse_extended_import_path(path_entry)
+        path = self._norm_path(path)
+        paths = self._glob_path(path) if self._is_glob(path) else [path]
+        if not paths:
+            if self.debug:
+                self.log.dbg("glob path {} didn't expand".format(path))
+            self._handle_non_existing_path(path, fatal_not_found)
+            return []
+
+        checked_paths = (self._check_path_existence(p, fatal_not_found)
+                         for p in paths)
+        return [p for p in checked_paths if p]
+
+    def _resolve_paths(self, paths):
+        """Resolve a list of path to existing paths.
+
+        This function resolves a list of paths. This means normalizing,
+        expanding globs and checking for existence, taking in account
+        fatal_not_found flags.
+
+        :param paths: A list of paths. Might contain globs and options.
+        :type paths: List of str
+        :return: A list of processed paths.
+        :rtype: List of str
+        """
+        processed_paths = (self._process_path(p) for p in paths)
+        return list(chain.from_iterable(processed_paths))
 
     def _import_actions(self):
         """import external actions from paths"""
         paths = self.settings.get(self.key_import_actions, None)
         if not paths:
             return
-        paths = self._glob_paths(paths)
-        for p in paths:
-            path, fatal_not_found = self._norm_extended_import_path(p)
+        paths = self._resolve_paths(paths)
+        for path in paths:
             if self.debug:
                 self.log.dbg('import actions from {}'.format(path))
             new = self._import_sub(path, self.key_actions,
                                    mandatory=False,
-                                   patch_func=self._norm_actions,
-                                   fatal_not_found=fatal_not_found)
+                                   patch_func=self._norm_actions)
             self.actions = self._merge_dict(new, self.actions)
 
     def _import_profiles_dotfiles(self):
@@ -626,26 +717,17 @@ class CfgYaml:
                 continue
             if self.debug:
                 self.log.dbg('import dotfiles for profile {}'.format(k))
-            paths = self._glob_paths(imp)
-            for p in paths:
+            paths = self._resolve_paths(imp)
+            for path in paths:
                 current = v.get(self.key_dotfiles, [])
-                path = self._norm_path(p)
                 new = self._import_sub(path, self.key_dotfiles,
                                        mandatory=False)
                 v[self.key_dotfiles] = new + current
 
     def _import_config(self, path):
         """import config from path"""
-        path, fatal_not_found = self._norm_extended_import_path(path)
         if self.debug:
             self.log.dbg('import config from {}'.format(path))
-        if not os.path.exists(path):
-            err = 'config path not found: {}'.format(path)
-            if fatal_not_found:
-                raise YamlException(err)
-            else:
-                self.log.warn(err)
-                return
         sub = CfgYaml(path, profile=self.profile, debug=self.debug)
 
         # settings are ignored from external file
@@ -677,22 +759,18 @@ class CfgYaml:
         imp = self.settings.get(self.key_import_configs, None)
         if not imp:
             return
-        paths = self._glob_paths(imp)
+        paths = self._resolve_paths(imp)
         for path in paths:
             self._import_config(path)
 
-    def _import_sub(self, path, key, mandatory=False,
-                    patch_func=None, fatal_not_found=True):
+    def _import_sub(self, path, key, mandatory=False, patch_func=None):
         """
         import the block "key" from "path"
         patch_func is applied to each element if defined
         """
         if self.debug:
             self.log.dbg('import \"{}\" from \"{}\"'.format(key, path))
-            self.log.dbg('ignore non existing: \"{}\"'.format(fatal_not_found))
-        extdict = self._load_yaml(path, fatal_not_found=fatal_not_found)
-        if extdict is None and not fatal_not_found:
-            return {}
+        extdict = self._load_yaml(path)
         new = self._get_entry(extdict, key, mandatory=mandatory)
         if patch_func:
             if self.debug:
@@ -740,6 +818,10 @@ class CfgYaml:
             self.dirty = True
         return self.dirty
 
+    def get_all_dotfile_keys(self):
+        """return all existing dotfile keys"""
+        return self.dotfiles.keys()
+
     def add_dotfile(self, key, src, dst, link):
         """add a new dotfile"""
         if key in self.dotfiles.keys():
@@ -902,16 +984,9 @@ class CfgYaml:
         """dump the config dictionary"""
         return self.yaml_dict
 
-    def _load_yaml(self, path, fatal_not_found=True):
+    def _load_yaml(self, path):
         """load a yaml file to a dict"""
         content = {}
-        if not os.path.exists(path):
-            err = 'config path not found: {}'.format(path)
-            if fatal_not_found:
-                raise YamlException(err)
-            else:
-                self.log.warn(err)
-                return None
         try:
             content = self._yaml_load(path)
         except Exception as e:
@@ -980,24 +1055,15 @@ class CfgYaml:
         return new
 
     def _is_glob(self, path):
-        """quick test if path is a glob"""
+        """Quick test if path is a glob."""
         return '*' in path or '?' in path
 
-    def _glob_paths(self, paths):
-        """glob a list of paths"""
-        if not isinstance(paths, list):
-            paths = [paths]
-        res = []
-        for p in paths:
-            if not self._is_glob(p):
-                res.append(p)
-                continue
-            p = os.path.expanduser(p)
-            new = glob.glob(p)
-            if not new:
-                raise YamlException('bad path: {}'.format(p))
-            res.extend(glob.glob(p))
-        return res
+    def _glob_path(self, path):
+        """Expand a glob."""
+        if self.debug:
+            self.log.dbg('expanding glob {}'.format(path))
+        expanded_path = os.path.expanduser(path)
+        return glob.glob(expanded_path, recursive=True)
 
     def _debug_vars(self, variables):
         """pretty print variables"""
@@ -1008,11 +1074,17 @@ class CfgYaml:
             self.log.dbg('\t\"{}\": {}'.format(k, v))
 
     def _norm_path(self, path):
-        """resolve a path either absolute or relative to config path"""
+        """Resolve a path either absolute or relative to config path"""
+        if self.debug:
+            self.log.dbg('normalizing path {}'.format(path))
         if not path:
             return path
         path = os.path.expanduser(path)
         if not os.path.isabs(path):
+            if self.debug:
+                self.log.dbg('normalizing path {} relative to config file '
+                             'directory'.format(path))
+
             d = os.path.dirname(self.path)
             return os.path.join(d, path)
         return os.path.normpath(path)
