@@ -52,7 +52,7 @@ USAGE = """
 
 Usage:
   dotdrop install   [-VbtfndDa] [-c <path>] [-p <profile>] [<key>...]
-  dotdrop import    [-Vbdf]     [-c <path>] [-p <profile>]
+  dotdrop import    [-Vbdf]     [-c <path>] [-p <profile>] [-s <path>]
                                 [-l <link>] <path>...
   dotdrop compare   [-Vb]       [-c <path>] [-p <profile>]
                                 [-C <file>...] [-i <pattern>...]
@@ -66,23 +66,24 @@ Usage:
   dotdrop --version
 
 Options:
-  -p --profile=<profile>  Specify the profile to use [default: {}].
+  -a --force-actions      Execute all actions even if no dotfile is installed.
   -c --cfg=<path>         Path to the config.
   -C --file=<path>        Path of dotfile to compare.
   -i --ignore=<pattern>   Pattern to ignore.
-  -l --link=<link>        "link_on_import" (nolink|link|link_children).
+  -l --link=<link>        Link option (nolink|link|link_children).
+  -p --profile=<profile>  Specify the profile to use [default: {}].
+  -s --as=<path>          Import as a different path from actual path.
+  -b --no-banner          Do not display the banner.
+  -d --dry                Dry run.
+  -D --showdiff           Show a diff before overwriting.
+  -f --force              Do not ask user confirmation for anything.
+  -G --grepable           Grepable output.
+  -k --key                Treat <path> as a dotfile key.
   -n --nodiff             Do not diff when installing.
+  -P --show-patch         Provide a one-liner to manually patch template.
   -t --temp               Install to a temporary directory for review.
   -T --template           Only template dotfiles.
-  -D --showdiff           Show a diff before overwriting.
-  -P --show-patch         Provide a one-liner to manually patch template.
-  -f --force              Do not ask user confirmation for anything.
-  -a --force-actions      Execute all actions even if no dotfile is installed.
-  -k --key                Treat <path> as a dotfile key.
   -V --verbose            Be verbose.
-  -d --dry                Dry run.
-  -b --no-banner          Do not display the banner.
-  -G --grepable           Grepable output.
   -v --version            Show version.
   -h --help               Show this screen.
 """.format(BANNER, PROFILE)
@@ -237,6 +238,7 @@ class Options(AttrMonitor):
         self.compare_ignore = uniq_list(self.compare_ignore)
         # "import" specifics
         self.import_path = self.args['<path>']
+        self.import_as = self.args['--as']
         # "update" specifics
         self.update_path = self.args['<path>']
         self.update_iskey = self.args['--key']
@@ -256,7 +258,7 @@ class Options(AttrMonitor):
         # variables
         self.variables = self.conf.get_variables()
         # the dotfiles
-        self.dotfiles = self.conf.get_dotfiles(self.profile)
+        self.dotfiles = self.conf.get_dotfiles()
         # the profiles
         self.profiles = self.conf.get_profiles()
 
